from .appointment import Appointment as Appointment
from .calendar_event import CalendarEvent as CalendarEvent
from .child import Child as Child
from .daily_overview import DailyOverview as DailyOverview
from .easyiq_homework import EasyIQHomework as EasyIQHomework
from .library import LibraryLoan as LibraryLoan
from .library import LibraryStatus as LibraryStatus
from .meebook_weekplan import MeebookStudentPlan as MeebookStudentPlan
from .message import Message as Message
from .message_thread import MessageThread as MessageThread
from .momo_huskeliste import MomoUserCourses as MomoUserCourses
from .momo_huskeliste import UserReminders as UserReminders
from .mu_task import MUTask as MUTask
from .mu_weekly_letter import MUWeeklyPerson as MUWeeklyPerson
from .notification import Notification as Notification
from .post import Post as Post
from .presence_template import DayTemplate as DayTemplate
from .presence_template import PresenceWeekTemplate as PresenceWeekTemplate
from .presence_template import SpareTimeActivity as SpareTimeActivity
from .profile import Profile as Profile
from .widget import WidgetConfiguration as WidgetConfiguration
